#!/usr/bin/python
#
#   Copyright 2011 (c) ONELAN Ltd. www.onelan.co.uk
#   All rights reserved.
#
#
#   main.py
#
import sys
import os

'''
proxy basic auth
proxy digest auth
proxy no auth
basic auth
digest auth
no auth

with 

http port 80 digest auth
http port 80 basic auth
http port 80 no auth
http port 8080 digest auth
http port 8080 basic auth
http port 8080 no auth
https port 443 digest auth
https port 443 basic auth
https port 443 no auth
https port 8443 digest auth
https port 8443 basic auth
https port 8443 no auth

Hosts:
    V8 NTB listening on ports 80, 8080, 443 and 8443
    V7 NTB listening on ports 80, 8080, 443 and 8443
    No auth host listening on ports 80, 8080, 443 and 8443
'''

def main( argv ):
    if len(argv) == 1:
        basic_host = 'ntb6747'; basic_title = 'NTB 6747 Home'
        digest_host = 'ntb9737'; digest_title = 'NTB 9737 Home'
        noauth_host = 'bugtracker'; noauth_title = 'Bug Tracker'

    elif len(argv) == 7:
        basic_host = argv[1]; basic_title = argv[2]
        digest_host = argv[3]; digest_title = argv[4]
        noauth_host = argv[5]; noauth_title = argv[6]

    else:
        print 'Usage: %s <basic-auth-host> <basic-auth-title> <digest-auth-host> <digest-auth-title> <noauth-host> <noauth-title>'
        return 1

    
    count = 0
    tests_in_error = []

    for proxy, proxy_username, proxy_password in \
            [(None, None, None)
            ,('http://testproxy:3130', None, None)
            ,('http://testproxy:3128', 'basicuser', 'basicpass')
            ,('http://testproxy:3129', 'digestuser', 'digestpass')]:
        for protocol, port in \
                [('http', None)
                ,('http', 80)
                ,('http', 8080)
                ,('https', None)
                ,('https', 443)
                ,('https', 8443)]:
            for host, expected_title, username, password in \
                    [(noauth_host, noauth_title, None, None)
                    ,(basic_host, basic_title, 'remote', '9999')
                    ,(digest_host, digest_title, 'remote', '9999')]:
                cmd = ['%s ./urlget.py' % (sys.executable,)]
                url = '%s://%s' % (protocol, host)
                if port is not None:
                    url = url + ':%d' % (port,)
                cmd.append( url )
                cmd.append( '--timeout 60' )
                if username is not None:
                    cmd.append( '--auth %s %s' % (username, password) )
                if proxy is not None:
                    cmd.append( '--proxy %s' % (proxy,) )
                    if proxy_username is not None:
                        cmd.append( '--proxy-auth %s %s' % (proxy_username, proxy_password) )
                cmd.append( '--expected-title "%s"' % (expected_title,) )

                count += 1
                cmd_line = ' '.join( cmd )
                print 'Test:',count, cmd_line
                rc = os.system( cmd_line )
                if rc != 0:
                    tests_in_error.append( (count, cmd_line) )

    if len(tests_in_error) == 0:
        print 'Info: All %d tests passed' % (count,)

    else:
        print 'Error: %d or %d tests failed' % (len(tests_in_error), count)
        for index, cmd_line in tests_in_error:
            print 'Error: %d failed: %s' % (index, cmd_line)
    return 0

if __name__ == '__main__':
    sys.exit( main( sys.argv ) )
