#
#    Copyright 2005-2011 (c) ONELAN Ltd. www.onelan.co.uk
#    All rights reserved.
#
import sys
import urlparse
import socket

import httplib
import urllib2
import urllib

import encodings.idna   # force in for urllib to use
type(encodings.idna)

class UrlGet:
    def __init__( self, cookie_jar=None, add_headers = None ):
        self.debug = False
        self.username = None
        self.password = None

        self.proxy_url = None
        self.proxy_username = None
        self.proxy_password = None
        self.proxy_allow_cache = True

        self.is_server_error = None
        self.is_file_error = None

        self.error_type = None
        self.error_detail = None
        self.error_reason = None
        self.error_code = None

        self.range_start = None

        self.cookie_jar = cookie_jar
        self.add_headers = add_headers

        self.response = None
        
        self.__timeout = None
        self.protocol = None

    def setDebug( self, value ):
        self.debug = value
        if self.debug:
            print 'Debug: Enabled'

    def setCredentials( self, username, password ):
        self.username = username
        self.password = password

    def setProxyCredentials( self, username, password ):
        self.proxy_username = username
        self.proxy_password = password

    def setProxy( self, url ):
        assert url.startswith( 'http://' )
        self.proxy_url = url

    def setAllowCache( self, enable=True ):
        self.proxy_allow_cache = enable

    def setRangeStart( self, range_start ):
        self.range_start = range_start

    def isRangeSupported( self ):
        return self.range_start is not None and self.response.code == 206
        
    def setTimeout( self, timeout ):
        self.__timeout = timeout

    def open( self, url, put_req_data=None ):
        assert self.response is None
        self.protocol, addr, path, Q, Q = urlparse.urlsplit( url, 'http' )

        # set up authentication info
        handlers = []
        if self.protocol == 'ftp':
            if self.username is not None:
                url = 'ftp://%s:%s@%s%s' % (self.username, self.password, addr, path)

        elif self.protocol in ['http','https']:
            if self.username is not None:
                if self.debug:
                    print 'Debug: Add host password for %s as %s %s' % (addr, self.username, self.password)

                password_manager = urllib2.HTTPPasswordMgrWithDefaultRealm()
                password_manager.add_password( None, addr, self.username, self.password )

                handlers.append( urllib2.HTTPBasicAuthHandler( password_manager ) )
                handlers.append( urllib2.HTTPDigestAuthHandler( password_manager ) )

        else:
            self.error_type = 'URL Error'
            self.error_code = 200
            self.error_reason = '%s cannot handle protocol: %s' % ('NTB', self.protocol)
            return False

        if self.proxy_url is not None:
            if self.debug:
                print 'Debug: ProxyHandler( %r: %r )' % (self.protocol, self.proxy_url)

            handlers.append( urllib2.ProxyHandler( {self.protocol: self.proxy_url} ) )
            if self.proxy_username is not None:
                Q, proxy_addr, Q, Q, Q = urlparse.urlsplit( self.proxy_url, 'http' )

                if self.debug:
                    print 'Debug: Add proxy password for %s as %s %s' % (proxy_addr, self.username, self.password)

                password_manager = urllib2.HTTPPasswordMgrWithDefaultRealm()
                password_manager.add_password( None, proxy_addr, self.proxy_username, self.proxy_password )
                #password_manager.add_password( None, addr, self.proxy_username, self.proxy_password )

                handlers.append( urllib2.ProxyDigestAuthHandler( password_manager ) )
                handlers.append( urllib2.ProxyBasicAuthHandler( password_manager ) )

        # build a new opener that adds authentication
        opener = urllib2.build_opener( *handlers )
        if self.debug:
            opener.set_debuglevel( 2 )

        # Replace the user agent string. Some sites (google for example)
        # reject access from programs like curl and python's urllib
        all_add_headers = [('User-Agent', 'Mozilla/5.0 (X11; U; Linux i686; en-US; rv:1.8) Gecko/20051111 Firefox/1.5')]

        if self.add_headers is not None:
            for (key,val) in self.add_headers:
                if key.lower() == 'user-agent':
                    all_add_headers = []  # use value from self.add_headers, not default version from above
                    break
            all_add_headers.extend( self.add_headers )

        req = urllib2.Request( url, data=put_req_data )

        for (key,val) in all_add_headers:
            req.add_header( key, val )

        # add any cookies that have been set
        if self.cookie_jar is not None:
            self.cookie_jar.add_cookie_header( req )

        if self.proxy_url is not None and not self.proxy_allow_cache:
            if self.debug:
                print 'Debug: adding Cache-control header'
            req.add_header( 'Cache-Control', 'max-age=0' )

        if self.range_start is not None:
            if self.debug:
                print 'Debug: adding Range header'
            req.add_header( 'Range', 'bytes=%d-' % self.range_start )

        self.is_file_error = False
        self.is_server_error = False

        try:
            if self.__timeout is not None:
                if self.debug: 
                    print 'Debug: Set timeout to: %s seconds' % self.__timeout
                self.response = opener.open( req, timeout=self.__timeout )
            else:
                self.response = opener.open( req )

            if self.cookie_jar is not None:
                self.cookie_jar.extract_cookies( self.response, req )

            return True

        except socket.error, detail:
            self.error_type = 'Network Error'
            self.error_detail = detail
            self.error_reason = str(detail)
            return False

        except socket.timeout, detail:
            self.error_type = 'Timeout'
            self.error_detail = detail
            self.error_reason = 'Server not responding'
            return False

        except (ValueError,TypeError,IndexError), detail:
            self.error_type = 'HTTP Error'
            self.error_detail = detail
            self.error_reason = str(detail)
            self.error_code = 500
            
            return False

        except urllib2.HTTPError, detail:
            if self.debug:
                print 'Debug: HTTPError detail',repr(detail),dir(detail)
                print 'Debug: HTTPError code',repr(detail.code)
            # affect this file on server only
            self.is_file_error = True

            self.error_type = 'HTTP Server Error'
            self.error_detail = detail
            self.error_reason = detail.msg
            self.error_code = detail.code
            if self.error_reason == '':
                self.error_reason = 'Code %d' % self.error_code
            else:
                self.error_reason = 'Code %d - %s' % (self.error_code, self.error_reason)
            return False
            
        except httplib.BadStatusLine, detail:
            #if self.debug: traceback.print_exc(file=sys.stdout)
            print repr(detail)
            print dir(detail)
            print repr(detail.args)
            self.error_type = 'HTTP Exception'
            self.error_detail = detail
            self.error_reason = 'Status: %r: %r' % (detail.args, detail.line)
            return False

        except httplib.HTTPException, detail:
            #if self.debug: traceback.print_exc(file=sys.stdout)
            print repr(detail)
            print dir(detail)
            print repr(detail.args)
            self.error_type = 'HTTP Exception'
            self.error_detail = detail
            if len(detail.args) > 0:
                self.error_reason = str(detail.args[0])
            else:
                self.error_reason = 'no reason given'
            return False

        except urllib2.URLError, detail:
            #if self.debug: traceback.print_exc(file=sys.stdout)
            
            self.error_type = 'URL Error'
            self.error_detail = detail

            if self.debug:
                print 'Debug: URLError detail',repr(detail),dir(detail)
            if hasattr( detail, 'reason' ) and hasattr( detail.reason, 'args' ):
                if self.debug:
                    print 'Debug: URLError detail.reason',repr(detail.reason),dir(detail.reason)
                    print 'Debug: URLError detail.args',repr(detail.reason.args)
                self.error_detail = detail.reason
                self.error_reason = detail.reason.args[-1]

                if not isinstance( self.error_reason, unicode ):
                    try:
                        self.error_reason = self.error_reason.decode( 'utf-8' )
                    except:
                        self.error_reason = repr( self.error_reason )

            else:
                self.error_reason = repr( detail )

            if hasattr( detail, 'code' ):
                if self.debug:
                    print 'Debug: URLError detail.code',repr(detail.code)
                self.error_code = detail.code

            else:
                self.error_code = 500

            if self.error_code in [404]:
                # affect this file on server only
                self.is_file_error = True
            else:
                # affect all files on server
                self.is_server_error = True

            if self.error_reason == '':
                self.error_reason = 'URL Error code %d' % self.error_code
            return False

        except IOError, detail:
            if( hasattr( detail, 'args' ) 
            and detail.args[0] == 'ftp error' ):
                self.error_type = 'FTP Error'

                detail = detail.args[1]
                self.error_detail = detail
                # detail will be a string in some cases (seen in a customer traceback report)
                if hasattr( detail, 'args' ) and len(detail.args) > 0:
                    self.error_reason = detail.args[-1]
                else:
                    self.error_reason = ''
                self.error_code = self.error_reason[0:3]
                if self.error_code in ['450']:
                    self.is_file_error = True
                else:
                    self.is_server_error = True
                return False

            else:
                self.error_type = 'IO Error'
                self.error_detail = detail
                self.error_reason = str(detail)
                self.error_code = 500
                return False

    def read( self, size=None ):
        assert self.response is not None
        
        try:
            self.error_type = None
            if size is None:
                return self.response.read()
            else:
                return self.response.read( size )

        except socket.error, detail:
            self.error_type = 'Read Error'
            self.error_detail = detail
            self.error_reason = str(detail)

            if self.debug:
                print ( 'Debug: Read failed.  Type: %s, detail: %s, error_reason: %s' 
                    % (self.error_type, self.error_detail, self.error_reason) )

            return ''

    def close( self ):
        if self.response is not None:
            self.response.close()
            self.response = None

    def isServerError( self ):
        return self.is_server_error

    def isFileError( self ):
        return self.is_file_error

def test( argv ): #pragma: no cover
    try:
        site_url = argv[0]
        username = None
        password = None
        proxy_url = None
        proxy_username = None
        proxy_password = None
        expected_title = None

        get = UrlGet()

        arg = 1
        while arg < len(argv):
            if argv[arg] == '--auth':
                username = argv[arg+1]
                password = argv[arg+2]
                get.setCredentials( username, password )
                arg += 3

            elif argv[arg] == '--proxy':
                proxy_url = argv[arg+1]
                get.setProxy( proxy_url )
                arg += 2

            elif argv[arg] == '--range-start':
                range_start = int( argv[arg+1] )
                get.setRangeStart( range_start )
                arg += 2

            elif argv[arg] == '--proxy-auth':
                proxy_username = argv[arg+1]
                proxy_password = argv[arg+2]
                get.setProxyCredentials( proxy_username, proxy_password )
                arg += 3

            elif argv[arg] == '--reload':
                get.setAllowCache( False )
                arg += 1

            elif argv[arg] == '--debug':
                get.setDebug( True )
                arg += 1

            elif argv[arg] == '--timeout':
                timeout = float( argv[arg+1] )
                get.setTimeout( timeout )
                arg += 2

            elif argv[arg] == '--expected-title':
                expected_title = argv[arg+1]
                arg += 2
                
            else:
                print 'Arg %d Unknown "%s"' % (arg, argv[arg])
                raise IndexError

    except IndexError:
        print 'Usage: %s <url> [--auth <username> <password>] [--proxy <url> [--proxy-auth <username> <password>]] [--reload] [--debug] [--timeout <seconds>]' % __name__
        return 1

    if get.open( site_url ):
        if hasattr( get.response, 'code' ):
            print 'Info: Open OK (%d)' % get.response.code
        else:
            print 'Info: Open OK'

        if expected_title is None:
            print get.read()
            return 0

        else:
            result = get.read()
            title = result.split( '<title>' )[1].split( '</title>' )[0]
            if title == expected_title:
                print 'Info: title is "%r"' % (title,)
                return 0

            else:
                print 'Error:  expecting title is "%r"' % (expected_title,)
                print 'Error: but actual title is "%r"' % (title,)
                return 2
    else:
        print 'Error: Open Failed'

        if get.isServerError():
            print 'Error: Server Error'

        elif get.isFileError():
            print 'Error: File Error'

        print 'Error: type -',get.error_type
        print 'Error: detail -',get.error_detail
        print 'Error: reason -',get.error_reason
        print 'Error: code -',get.error_code
        return 1

if __name__ == '__main__': #pragma: no cover
    sys.exit( test( sys.argv[1:] ) )
